# -----------------------------------------------------------------------------
# R code for fitting bivariate circular model with Kato-Jones marginals and 
# Model 2 circula (density (11) in paper), after initial analysis completed.
# Antimode used as origen for marginal distribiution functions.
# -----------------------------------------------------------------------------

# -----------------------------------------------------------------------------
# MM estimation of gamma and rho for posited Model 2 circula.
# -----------------------------------------------------------------------------

mmgam <- Mod(sum(exp((1i)*(cang1-q*cang2)))/n) ; mmgam  
if (mmgam >= 1) mmgam <- 0.99
mmgam
mmrho <- Mod(sum(exp((1i)*(cang1-2*q*cang2)))/n)/mmgam ; mmrho  
if (mmrho <= (2*mmgam-1)) mmrho <- 2*mmgam-1+0.01
if (mmrho >= 1) mmrho <- 0.99
mmrho

# -----------------------------------------------------------------------------
Mod2cden <- function(gamma, rho, q, theta1, theta2) {
# -----------------------------------------------------------------------------

# Computes values of Model 2 circula density.

arg <- cos(theta1-q*theta2)
cden <- (1+2*gamma*(arg-rho)/(1+rho^2-2*rho*arg))/(4*pi*pi)
return(cden) 
}

# -----------------------------------------------------------------------------
Mod2ML <- function(q, mmgam, mmrho, ps1, ps2) {
# -----------------------------------------------------------------------------

# ML estimation of the parameters gamma and rho of a posited Model 2 circula 
# using MM estimates as starting value.

Mod2nll <- function(p, q, ps1, ps2){
gam <- p[1] ; mult <- p[2]
llrho <- 2*gam-1 ; rngrho <- 0.9999-llrho ; rho <- llrho+mult*rngrho
nll <- -sum(log(Mod2cden(gam, rho, q, ps1, ps2)))
return(nll) 
}

rholl <- 2*mmgam-1 ; rhorng <- 0.9999-rholl ; multmm <- (mmrho-rholl)/rhorng  
start <- c(mmgam, multmm) 
try <- nlminb(start, function(x) Mod2nll(x,q,ps1,ps2), lower=c(0,0),upper=c(0.9999,0.9999))
maxll <- -try[[2]] ; MLest <- try[[1]] ; MLgam <- MLest[1] ; MLmult <- MLest[2]
rholl <- 2*MLgam-1 ; rhorng <- 0.9999-rholl ; MLrho <- rholl+MLmult*rhorng 
return(list(maxll, MLgam, MLrho))
}

# -----------------------------------------------------------------------------

MLRes <- Mod2ML(q, mmgam, mmrho, cang1, cang2) 
maxll <- MLRes[[1]] ; MLgam <- MLRes[[2]] ; MLrho <- MLRes[[3]]
maxll ;  MLgam ; MLrho 
Mod2MLEs <- c(q, MLgam, MLrho) 

# -----------------------------------------------------------------------------
# Contour plot of fitted Model 2 circula with pseudo-values superimposed.
# -----------------------------------------------------------------------------

t1val <- seq(-pi, pi, 2*pi/200) ; t2val <- seq(-pi, pi, 2*pi/200)
nt <- length(t1val) ; circind <- 0 ; circ <- 0
for (i in 1:nt) { t2 <- t2val[i] 
for (j in 1:nt) { t1 <- t1val[j] ; circind <- circind+1
circ[circind] <- Mod2cden(MLgam, MLrho, q, t1, t2)
} }

par(lab=c(6,6,1), font.main=1, cex.axis=1.2, mai=c(0.45,0.45,0.05,0.05))
contour(seq(-pi, pi, 2*pi/200), seq(-pi, pi, 2*pi/200), matrix(circ,201,201), xlim=c(-pi, pi), ylim=c(-pi, pi), xaxt="n", yaxt="n", lty=1, lwd=2, col="darkblue", nlevels = 10, cex.lab=1.0, labcex=0.8)
axis(1, at=seq(-pi, pi, pi/2), labels=c(expression(-pi),expression(-pi/2),0,expression(pi/2),expression(pi)))
axis(2, at=seq(-pi, pi, pi/2), labels=c(expression(-pi),expression(-pi/2),0,expression(pi/2),expression(pi)))
points(cang1, cang2, pch=16, cex=0.6, col="red")

# -----------------------------------------------------------------------------
Mod2KJ4ML <- function(theta1, theta2, M1IN, M2IN, CIN) {
# -----------------------------------------------------------------------------

# Full ML estimation of the parameters of a posited bivariate circular model with 
# Kato-Jones marginals and a Model 2 circula, using the previously calculated ML 
# estimates as starting values.

q <- CIN[1] ; gamCin <- CIN[2] ; rhoCin <- CIN[3]
rholl <- 2*gamCin-1 ; rhorng <- 0.9999-rholl ; multCin <- (rhoCin-rholl)/rhorng
mu1in <- M1IN[1] ; gam1in <- M1IN[2] ; alpha21in <- M1IN[3] ; beta21in <- M1IN[4]
mu2in <- M2IN[1] ; gam2in <- M2IN[2] ; alpha22in <- M2IN[3] ; beta22in <- M2IN[4]
phi1in <- atan2(beta21in, (alpha21in-gam1in^2)) ; omega1in <- beta21in/(gam1in*(1-gam1in)*sin(phi1in))
phi2in <- atan2(beta22in, (alpha22in-gam2in^2)) ; omega2in <- beta22in/(gam2in*(1-gam2in)*sin(phi2in))
svals <- c(gamCin, multCin, mu1in, gam1in, phi1in, omega1in, mu2in, gam2in, phi2in, omega2in)

nllfn <- function(p, q, theta1, theta2){
gamC <- p[1] ; multC <- p[2]
mu1 <- p[3] ; gam1 <- p[4] ; phi1 <- p[5] ; omega1 <- p[6]
mu2 <- p[7] ; gam2 <- p[8] ; phi2 <- p[9] ; omega2 <- p[10]

n <- length(theta1)
alpha21 <- gam1*(gam1+(1-gam1)*omega1*cos(phi1)) ; beta21 <- gam1*(1-gam1)*omega1*sin(phi1)
ll1 <- sum(log(1+2*gam1^2*(gam1*cos(theta1-mu1)-alpha21)/(gam1^2+alpha21^2+beta21^2-2*gam1*(alpha21*cos(theta1-mu1)+beta21*sin(theta1-mu1)))))
nll1 <- -ll1+n*log(2*pi)
KJ4T1 <- c(mu1, gam1, alpha21, beta21)  

alpha22 <- gam2*(gam2+(1-gam2)*omega2*cos(phi2)) ; beta22 <- gam2*(1-gam2)*omega2*sin(phi2)
ll2 <- sum(log(1+2*gam2^2*(gam2*cos(theta2-mu2)-alpha22)/(gam2^2+alpha22^2+beta22^2-2*gam2*(alpha22*cos(theta2-mu2)+beta22*sin(theta2-mu2)))))
nll2 <- -ll2+n*log(2*pi)
KJ4T2 <- c(mu2, gam2, alpha22, beta22)

cang1 <- CircVals(theta1, KJ4T1) ; cang2 <- CircVals(theta2, KJ4T2)

rholl <- 2*gamC-1 ; rhorng <- 0.9999-rholl ; rhoC <- rholl+multC*rhorng 
nll3 <- -sum(log(Mod2cden(gamC, rhoC, q, cang1, cang2)))
nll <- nll1+nll2+nll3-(n*log(4*pi^2))
return(nll)
}

clo <- c(0,0,-pi,0.0001,-pi,0.0001,-pi,0.0001,-pi,0.0001)
chi <- c(0.9999,0.9999,pi,0.9999,pi,0.9999,pi,0.9999,pi,0.9999)
mlres <- nlminb(svals, function(x) nllfn(x,q,theta1,theta2),lower=clo, upper=chi)
maxi <- -mlres[[2]] ; pests <- mlres[[1]]

gamC <- pests[1] ; multC <- pests[2]
rholl <- 2*gamC-1 ; rhorng <- 0.9999-rholl ; rhoC <- rholl+multC*rhorng

mu1 <- Arg(exp((1i)*pests[3])) ; gam1 <- pests[4] 
phi1 <- Arg(exp((1i)*pests[5])) ; omega1 <- pests[6]
alpha21 <- gam1^2+gam1*(1-gam1)*omega1*cos(phi1) ; beta21 <- gam1*(1-gam1)*omega1*sin(phi1)

mu2 <- Arg(exp((1i)*pests[7])) ; gam2 <- pests[8]
phi2 <- Arg(exp((1i)*pests[9])) ; omega2 <- pests[10]
alpha22 <- gam2^2+gam2*(1-gam2)*omega2*cos(phi2) ; beta22 <- gam2*(1-gam2)*omega2*sin(phi2)

mleres <- c(maxi,gamC,rhoC,mu1,gam1,alpha21,beta21,mu2,gam2,alpha22,beta22)
return(mleres)
}

# -----------------------------------------------------------------------------

MLres <- Mod2KJ4ML(theta1, theta2, KJ4MLT1, KJ4MLT2, Mod2MLEs) ; MLres

maxll <- MLres[1] ; npar <- 10
AIC <- 2*npar-2*maxll ; AIC
BIC <- npar*log(n)-2*maxll ; BIC

# -----------------------------------------------------------------------------
Mod2KJden <- function(circarg, den1arg, den2arg, t1, t2) {
# -----------------------------------------------------------------------------

# Computes values of bivariate circular density with Kato-Jones 
# marginal circular densities and a Model 2 circula density. 

q <- circarg[1] ; gammac <- circarg[2] ; rhoc <- circarg[3]
mu1 <- den1arg[1] ; gamma1 <- den1arg[2] ; alpha21 <- den1arg[3] ; beta21 <- den1arg[4] 
rho1 <- sqrt(alpha21^2+beta21^2)/gamma1 ; lambda1 <- atan2(beta21,alpha21)
mu2 <- den2arg[1] ; gamma2 <- den2arg[2] ; alpha22 <- den2arg[3] ; beta22 <- den2arg[4] 
rho2 <- sqrt(alpha22^2+beta22^2)/gamma2 ; lambda2 <- atan2(beta22,alpha22)

nt <- length(t1) ; denind <- 0 ; den <- 0
p1 <- KJPDF(t1, mu1, rho1, gamma1, lambda1) ; arg1 <- CircVals(t1, den1arg)
p2 <- KJPDF(t2, mu2, rho2, gamma2, lambda2) ; arg2 <- CircVals(t2, den2arg)
for (i in 1:nt) { 
for (j in 1:nt) { denind <- denind+1
arg <- cos(arg1[j]-q*arg2[i])
circden <- 1+2*gammac*(arg-rhoc)/(1+rhoc^2-2*rhoc*arg)
den[denind] <- p1[j]*p2[i]*circden
} }
return(den) 
}

# -----------------------------------------------------------------------------
# Contour plot of ML fitted bivariate circular density with data superimposed. 
# -----------------------------------------------------------------------------

circML <- c(q,MLres[2],MLres[3])
mu1ML <- MLres[4] ; gam1ML <- MLres[5] ; a21ML <- MLres[6] ; b21ML <- MLres[7]
den1ML <- c(mu1ML, gam1ML, a21ML, b21ML)
mu2ML <- MLres[8] ; gam2ML <- MLres[9] ; a22ML <- MLres[10] ; b22ML <- MLres[11]
den2ML <- c(mu2ML, gam2ML, a22ML, b22ML)

t1val <- seq(-pi, pi, 2*pi/800) ; t2val <- seq(-pi, pi, 2*pi/800)
den <- Mod2KJden(circML, den1ML, den2ML, t1val, t2val)

par(lab=c(6,6,1), font.main=1, cex.axis=1.2, mai=c(0.45,0.45,0.05,0.05))
contour(seq(-pi, pi, 2*pi/800), seq(-pi, pi, 2*pi/800), matrix(den,801,801), xlim=c(-pi, pi), ylim=c(-pi, pi), xaxt="n", yaxt="n", lty=1, lwd=2, col="darkblue", nlevels = 15, cex.lab=1.0, labcex=0.8)
axis(1, at=seq(-pi, pi, pi/2), labels=c(expression(-pi),expression(-pi/2),0,expression(pi/2),expression(pi)))
axis(2, at=seq(-pi, pi, pi/2), labels=c(expression(-pi),expression(-pi/2),0,expression(pi/2),expression(pi)))
points(theta1, theta2, pch=16, cex=0.6, col="red")

maxden <- max(den) ; maxden

# -----------------------------------------------------------------------------
# Scatterplot of the pseudo-sample using the ML estimates of the marginal 
# distributions from the full bivariate circular model fit.
# -----------------------------------------------------------------------------

cang1 <- CircVals(theta1, den1ML) ; cang2 <- CircVals(theta2, den2ML)

par(mfrow=c(1,1), lab=c(6,6,1), font.main=1, cex.axis=1.2, mai=c(0.45,0.45,0.05,0.05))
plot(cang1, cang2, pch=16, cex=0.6, col="red", xlim=c(-pi, pi), ylim=c(-pi, pi), xaxt="n", yaxt="n")
axis(1, at=seq(-pi, pi, pi/2), labels=c(expression(-pi),expression(-pi/2),0,expression(pi/2),expression(pi)))
axis(2, at=seq(-pi, pi, pi/2), labels=c(expression(-pi),expression(-pi/2),0,expression(pi/2),expression(pi)))

# -----------------------------------------------------------------------------
# Contour plot of fitted Model 2 circula from the full model fit, with 
# pseudo-values superimposed.
# -----------------------------------------------------------------------------

mlgamC <- MLres[2] ; mlrhoC <- MLres[3]

t1val <- seq(-pi, pi, 2*pi/200) ; t2val <- seq(-pi, pi, 2*pi/200)
nt <- length(t1val) ; circind <- 0 ; circ <- 0
for (i in 1:nt) { t2 <- t2val[i] 
for (j in 1:nt) { t1 <- t1val[j] ; circind <- circind+1
circ[circind] <- Mod2cden(mlgamC, mlrhoC, q, t1, t2)
} }

par(lab=c(6,6,1), font.main=1, cex.axis=1.2, mai=c(0.45,0.45,0.05,0.05))
contour(seq(-pi, pi, 2*pi/200), seq(-pi, pi, 2*pi/200), matrix(circ,201,201), xlim=c(-pi, pi), ylim=c(-pi, pi), xaxt="n", yaxt="n", lty=1, lwd=2, col="darkblue", nlevels = 10, cex.lab=1.0, labcex=0.8)
axis(1, at=seq(-pi, pi, pi/2), labels=c(expression(-pi),expression(-pi/2),0,expression(pi/2),expression(pi)))
axis(2, at=seq(-pi, pi, pi/2), labels=c(expression(-pi),expression(-pi/2),0,expression(pi/2),expression(pi)))
points(cang1, cang2, pch=16, cex=0.6, col="red")

# -----------------------------------------------------------------------------
# Calculation of MM estimate of rhoR, identification of q and bootstrap testing 
# for independence.
# -----------------------------------------------------------------------------

n <- length(cang1)
p1 <- abs(sum(exp((1i)*(cang1-cang2)))) ; p2 <- abs(sum(exp((1i)*(cang1+cang2))))
rhoRest <- (p1-p2)/n ; arhoRest <- abs(rhoRest) ; rhoRest
qval <- sign(rhoRest) ; qval

nind <- 1
B <- 999
for (i in 1:B) {
rcang2 <- sample(cang2,replace=F)
p1 <- abs(sum(exp((1i)*(cang1-rcang2)))) ; p2 <- abs(sum(exp((1i)*(cang1+rcang2))))
rhoRestB <- (p1-p2)/n
if (abs(rhoRestB) >= arhoRest) nind <- nind+1
}
pvalest <- nind/(B+1) ; pvalest

# -----------------------------------------------------------------------------
# Goodness-of-fit testing based on transformation to putative toroidal pseudo-
# uniformity
# -----------------------------------------------------------------------------

# -----------------------------------------------------------------------------
# Model 2 circula conditional densities.
# -----------------------------------------------------------------------------

# -----------------------------------------------------------------------------
Mod2cden1g2 <- function(theta1, theta2, gamma, rho, q) {
# -----------------------------------------------------------------------------
arg <- cos(theta1-q*theta2)
cden <- (1+2*gamma*(arg-rho)/(1+rho^2-2*rho*arg))/(2*pi)
return(cden) 
}

# -----------------------------------------------------------------------------
Mod2cden2g1 <- function(theta2, theta1, gamma, rho, q) {
# -----------------------------------------------------------------------------
arg <- cos(theta1-q*theta2)
cden <- (1+2*gamma*(arg-rho)/(1+rho^2-2*rho*arg))/(2*pi)
return(cden) 
}

# -----------------------------------------------------------------------------
# Computation of putative toroidal pseudo-uniform variates.
# -----------------------------------------------------------------------------

arg1 <- 0 ; arg2 <- 0 ; arg3 <- 0 ; arg4 <- 0
for (i in 1:n) {
arg1[i] <- cang1[i]%%(2*pi) ; arg2[i] <- cang2[i]%%(2*pi)
a1 <- arg1[i] ; a2 <- arg2[i]
arg3[i] <- 2*pi*integrate(function(x) Mod2cden1g2(x, a2, mlgamC, mlrhoC, q),0,a1,abs.tol=1e-50)[[1]]
arg4[i] <- 2*pi*integrate(function(x) Mod2cden2g1(x, a1, mlgamC, mlrhoC, q),0,a2,abs.tol=1e-50)[[1]]
arg1[i] <- Arg(exp((1i)*arg1[i])) ; arg2[i] <- Arg(exp((1i)*arg2[i]))
arg3[i] <- Arg(exp((1i)*arg3[i])) ; arg4[i] <- Arg(exp((1i)*arg4[i]))
}

# -----------------------------------------------------------------------------
# Scatterplots of putative toroidal pseudo-uniform variates.
# -----------------------------------------------------------------------------

par(lab=c(6,6,1), font.main=1, cex.axis=1.2, mai=c(0.45,0.45,0.05,0.05))
plot(arg2, arg3, pch=16, cex=0.6, col="red", xlim=c(-pi, pi), ylim=c(-pi, pi), xaxt="n", yaxt="n")
axis(1, at=seq(-pi, pi, pi/2), labels=c(expression(-pi),expression(-pi/2),0,expression(pi/2),expression(pi)))
axis(2, at=seq(-pi, pi, pi/2), labels=c(expression(-pi),expression(-pi/2),0,expression(pi/2),expression(pi)))

par(lab=c(6,6,1), font.main=1, cex.axis=1.2, mai=c(0.45,0.45,0.05,0.05))
plot(arg1, arg4, pch=16, cex=0.6, col="red", xlim=c(-pi, pi), ylim=c(-pi, pi), xaxt="n", yaxt="n")
axis(1, at=seq(-pi, pi, pi/2), labels=c(expression(-pi),expression(-pi/2),0,expression(pi/2),expression(pi)))
axis(2, at=seq(-pi, pi, pi/2), labels=c(expression(-pi),expression(-pi/2),0,expression(pi/2),expression(pi)))

# -----------------------------------------------------------------------------
WellnerBTStat <- function(arg1, arg2, arg3, arg4) {
# -----------------------------------------------------------------------------

# Computation of the maximum of the Wellner test statistic values for the 
# two sets of putative toroidal pseudo-uniform variates.

theta1 <- arg2 ; theta2 <- arg3
n <- length(theta1)
scc <- sum(cos(theta1)*cos(theta2)) ; scs <- sum(cos(theta1)*sin(theta2))
ssc <- sum(sin(theta1)*cos(theta2)) ; sss <- sum(sin(theta1)*sin(theta2))
statval1 <- 4*(scc*scc+scs*scs+ssc*ssc+sss*sss)/n
theta1 <- arg1 ; theta2 <- arg4
scc <- sum(cos(theta1)*cos(theta2)) ; scs <- sum(cos(theta1)*sin(theta2))
ssc <- sum(sin(theta1)*cos(theta2)) ; sss <- sum(sin(theta1)*sin(theta2))
statval2 <- 4*(scc*scc+scs*scs+ssc*ssc+sss*sss)/n
statval <- max(statval1,statval2)
return(statval)
}

# -----------------------------------------------------------------------------

StatVals <- WellnerBTStat(arg1, arg2, arg3, arg4) ; StatVals

# -----------------------------------------------------------------------------
Mod2KJSim <- function(circarg, den1arg, den2arg, maxden, n) {
# -----------------------------------------------------------------------------

# Simulation from bivariate circular model with Kato-Jones marginal circular 
# densities and a Model 2 circula density, using simple acceptance-rejection. 

theta1 <- 0 ; theta2 <- 0
for (i in 1:n) {
stopgo <- 1
while (stopgo == 1) {
t1 <- 2*pi*runif(1,0,1)-pi ; t2 <- 2*pi*runif(1,0,1)-pi
boxh <- maxden*runif(1,0,1)
dval <- Mod2KJden(circarg, den1arg, den2arg, t1, t2)
if (boxh <= dval) {
stopgo <- 0
theta1[i] <- t1 ; theta2[i] <- t2
} } } 
return(list(theta1,theta2))
}

# -----------------------------------------------------------------------------
Mod2cSim <- function(gamma, rho, q, n) {
# -----------------------------------------------------------------------------

# Simulation from Model 2 circula using simple acceptance-rejection. 

cmax <- (1+2*gamma*(1-rho)/(1+rho^2-2*rho))/(4*pi^2) ; theta1 <- 0 ; theta2 <- 0
for (i in 1:n) {
stopgo <- 1
while (stopgo == 1) {
t1 <- 2*pi*runif(1,0,1)-pi ; t2 <- 2*pi*runif(1,0,1)-pi
boxh <- cmax*runif(1,0,1)
dval <- Mod2cden(gamma, rho, q, t1, t2) 
if (boxh <= dval) {
stopgo <- 0
theta1[i] <- t1 ; theta2[i] <- t2
} } } 
return(list(theta1,theta2))
}


# -----------------------------------------------------------------------------
BootGOF <- function(circML, den1ML, den2ML, maxden, n, B, StatVals) {
# -----------------------------------------------------------------------------

# Bootstrap goodness-of-fit testing based on the maximum of the Wellner test 
# statistic values for two sets of toroidal pseudo-uniform variates.

nextrmB <- 1 ; q <- circML[1]

for (j in 2:(B+1)) {

SimRes <- Mod2KJSim(circML, den1ML, den2ML, maxden, n)
thet1 <- SimRes[[1]] ; thet2 <- SimRes[[2]]
MLres <- Mod2KJ4ML(thet1, thet2, den1ML, den2ML, circML)
mu1ML <- MLres[4] ; gam1ML <- MLres[5] ; a21ML <- MLres[6] ; b21ML <- MLres[7]
d1ML <- c(mu1ML, gam1ML, a21ML, b21ML)
mu2ML <- MLres[8] ; gam2ML <- MLres[9] ; a22ML <- MLres[10] ; b22ML <- MLres[11]
d2ML <- c(mu2ML, gam2ML, a22ML, b22ML)
cang1 <- CircVals(thet1, d1ML) ; cang2 <- CircVals(thet2, d2ML)

mlgamC <- MLres[2] ; mlrhoC <- MLres[3]
arg1 <- 0 ; arg2 <- 0 ; arg3 <- 0 ; arg4 <- 0
for (i in 1:n) {
arg1[i] <- cang1[i]%%(2*pi) ; arg2[i] <- cang2[i]%%(2*pi)
a1 <- arg1[i] ; a2 <- arg2[i]
arg3[i] <- 2*pi*integrate(function(x) Mod2cden1g2(x, a2, mlgamC, mlrhoC, q),0,a1,abs.tol=1e-50)[[1]]
arg4[i] <- 2*pi*integrate(function(x) Mod2cden2g1(x, a1, mlgamC, mlrhoC, q),0,a2,abs.tol=1e-50)[[1]]
arg1[i] <- Arg(exp((1i)*arg1[i])) ; arg2[i] <- Arg(exp((1i)*arg2[i]))
arg3[i] <- Arg(exp((1i)*arg3[i])) ; arg4[i] <- Arg(exp((1i)*arg4[i]))
}

StatVals[j] <- WellnerBTStat(arg1, arg2, arg3, arg4) 
if (StatVals[j] >= StatVals[1]) {nextrmB <- nextrmB+1}
}

pvalB <- nextrmB/(B+1) 
return(list(pvalB, StatVals))
}

# -----------------------------------------------------------------------------

B <- 99 ; n <- length(theta1)
BootGOFRes <- BootGOF(circML, den1ML, den2ML, maxden, n, B, StatVals)

BootGOFRes[[1]] ; tstatB <- BootGOFRes[[2]] ; tstatB
sort(tstatB)












