# -----------------------------------------------------------------------------
# R code for fitting bivariate circular model with Kato-Jones marginals and 
# Model 4 circula (density (13) in paper), after initial analysis completed.
# Antimode used as origen for marginal distribiution functions.
# -----------------------------------------------------------------------------

# -----------------------------------------------------------------------------
# MM estimation of gamma, rho1 and rho2 for posited Model 4 circula.
# -----------------------------------------------------------------------------

mmgam <- Mod(sum(exp((1i)*(cang1-q*cang2)))/n) ; mmgam  
mmrho1 <- Mod(sum(exp((1i)*(2*cang1-q*cang2)))/n)/mmgam ; mmrho1
if (mmrho1 >= 1) mmrho1 <- 0.9999
mmrho2 <- Mod(sum(exp((1i)*(cang1-2*q*cang2)))/n)/mmgam ; mmrho2 
if (mmrho2 >= 1) mmrho2 <- 0.9999

# -----------------------------------------------------------------------------
maxgam <- function(rho1,rho2) {
# -----------------------------------------------------------------------------
absdif <- abs(rho1-rho2) ; prod <- rho1*rho2
if (absdif <= prod) {
arg <- 1/((1-rho1^2)*(1-rho2^2)) ; gammax <- prod/(arg-1)
} else if (absdif > prod) {
if (rho1 > rho2) {
gammax <- (1-rho1)*(1+rho2)/2
} else if (rho1 < rho2) {
gammax <- (1+rho1)*(1-rho2)/2
} }
}

gammax <- maxgam(mmrho1,mmrho2)
if (mmgam >= gammax) mmgam <- gammax-0.01
mmgam 

# -----------------------------------------------------------------------------
Mod4cden <- function(gamma, rho1, rho2, q, theta1, theta2) {
# -----------------------------------------------------------------------------

# Computes values of Model 4 circula density.

arg <- cos(theta1-q*theta2)
cden <- (1+2*gamma*(arg-rho2*cos(theta1)-rho1*cos(theta2)+rho1*rho2)/((1+rho1^2-2*rho1*cos(theta1))*(1+rho2^2-2*rho2*cos(theta2))))/(4*pi*pi)
return(cden) 
}

# -----------------------------------------------------------------------------
Mod4ML <- function(q, mmgam, mmrho1, mmrho2, ps1, ps2) {
# -----------------------------------------------------------------------------

# ML estimation of the parameters gamma, rho1 and rho2 of a posited Model 4 circula 
# using MM estimates as starting value.

Mod4nll <- function(p, q, ps1, ps2){
mult <- p[1] ; rho1 <- p[2] ; rho2 <- p[3] ; gammax <- maxgam(rho1,rho2) ; gam <- mult*gammax 
nll <- -sum(log(Mod4cden(gam, rho1, rho2, q, ps1, ps2)))
return(nll) 
}

gammax <- maxgam(mmrho1,mmrho2) ; multmm <- mmgam/gammax  
start <- c(multmm, mmrho1, mmrho2) 
try <- nlminb(start, function(x) Mod4nll(x,q,ps1,ps2), lower=c(0,0.0001,0.0001),upper=c(0.9999,0.9999,0.9999))
maxll <- -try[[2]] ; MLest <- try[[1]] ; MLmult <- MLest[1] ; MLrho1 <- MLest[2] ; MLrho2 <- MLest[3]
gammax <- maxgam(MLrho1,MLrho2) ; MLgam <- MLmult*gammax
return(list(maxll, MLgam, MLrho1, MLrho2))
}

# -----------------------------------------------------------------------------

MLRes <- Mod4ML(q, mmgam, mmrho1, mmrho2, cang1, cang2) 
maxll <- MLRes[[1]] ; MLgam <- MLRes[[2]] ; MLrho1 <- MLRes[[3]] ; MLrho2 <- MLRes[[4]]
maxll ;  MLgam ; MLrho1 ; MLrho2 
Mod4MLEs <- c(q, MLgam, MLrho1, MLrho2) 

# -----------------------------------------------------------------------------
# Contour plot of fitted Model 4 circula with pseudo-values superimposed.
# -----------------------------------------------------------------------------

t1val <- seq(-pi, pi, 2*pi/200) ; t2val <- seq(-pi, pi, 2*pi/200)
nt <- length(t1val) ; circind <- 0 ; circ <- 0
for (i in 1:nt) { t2 <- t2val[i] 
for (j in 1:nt) { t1 <- t1val[j] ; circind <- circind+1
circ[circind] <- Mod4cden(MLgam, MLrho1, MLrho2, q, t1, t2)
} }

par(lab=c(6,6,1), font.main=1, cex.axis=1.2, mai=c(0.45,0.45,0.05,0.05))
contour(seq(-pi, pi, 2*pi/200), seq(-pi, pi, 2*pi/200), matrix(circ,201,201), xlim=c(-pi, pi), ylim=c(-pi, pi), xaxt="n", yaxt="n", lty=1, lwd=2, col="darkblue", nlevels = 30, cex.lab=1.0, labcex=0.8)
axis(1, at=seq(-pi, pi, pi/2), labels=c(expression(-pi),expression(-pi/2),0,expression(pi/2),expression(pi)))
axis(2, at=seq(-pi, pi, pi/2), labels=c(expression(-pi),expression(-pi/2),0,expression(pi/2),expression(pi)))
points(cang1, cang2, pch=16, cex=0.6, col="red")

# -----------------------------------------------------------------------------
Mod4KJ4ML <- function(theta1, theta2, M1IN, M2IN, CIN) {
# -----------------------------------------------------------------------------

# Full ML estimation of the parameters of a posited bivariate circular model with 
# Kato-Jones marginals and a Model 4 circula, using the previously calculated ML 
# estimates as starting values.

q <- CIN[1] ; gamCin <- CIN[2] ; rho1Cin <- CIN[3] ; rho2Cin <- CIN[4]
gammax <- maxgam(rho1Cin,rho2Cin) ; multCin <- gamCin/gammax
mu1in <- M1IN[1] ; gam1in <- M1IN[2] ; alpha21in <- M1IN[3] ; beta21in <- M1IN[4]
mu2in <- M2IN[1] ; gam2in <- M2IN[2] ; alpha22in <- M2IN[3] ; beta22in <- M2IN[4]
phi1in <- atan2(beta21in, (alpha21in-gam1in^2)) ; omega1in <- beta21in/(gam1in*(1-gam1in)*sin(phi1in))
phi2in <- atan2(beta22in, (alpha22in-gam2in^2)) ; omega2in <- beta22in/(gam2in*(1-gam2in)*sin(phi2in))
svals <- c(multCin, rho1Cin, rho2Cin, mu1in, gam1in, phi1in, omega1in, mu2in, gam2in, phi2in, omega2in)

nllfn <- function(p, q, theta1, theta2){

multC <- p[1] ; rho1C <- p[2] ; rho2C <- p[3]
gammax <- maxgam(rho1C,rho2C) ; gamC <- multC*gammax
mu1 <- p[4] ; gam1 <- p[5] ; phi1 <- p[6] ; omega1 <- p[7]
mu2 <- p[8] ; gam2 <- p[9] ; phi2 <- p[10] ; omega2 <- p[11]

n <- length(theta1)
alpha21 <- gam1*(gam1+(1-gam1)*omega1*cos(phi1)) ; beta21 <- gam1*(1-gam1)*omega1*sin(phi1)
ll1 <- sum(log(1+2*gam1^2*(gam1*cos(theta1-mu1)-alpha21)/(gam1^2+alpha21^2+beta21^2-2*gam1*(alpha21*cos(theta1-mu1)+beta21*sin(theta1-mu1)))))
nll1 <- -ll1+n*log(2*pi)
KJ4T1 <- c(mu1, gam1, alpha21, beta21)  

alpha22 <- gam2*(gam2+(1-gam2)*omega2*cos(phi2)) ; beta22 <- gam2*(1-gam2)*omega2*sin(phi2)
ll2 <- sum(log(1+2*gam2^2*(gam2*cos(theta2-mu2)-alpha22)/(gam2^2+alpha22^2+beta22^2-2*gam2*(alpha22*cos(theta2-mu2)+beta22*sin(theta2-mu2)))))
nll2 <- -ll2+n*log(2*pi)
KJ4T2 <- c(mu2, gam2, alpha22, beta22)

cang1 <- CircVals(theta1, KJ4T1) ; cang2 <- CircVals(theta2, KJ4T2)
nll3 <- -sum(log(Mod4cden(gamC, rho1C, rho2C, q, cang1, cang2)))
nll <- nll1+nll2+nll3-(n*log(4*pi^2))
return(nll)
}

clo <- c(0.0001,1e-10,1e-10,-pi,0.0001,-pi,0.0001,-pi,0.0001,-pi,0.0001)
chi <- c(0.9999,0.9999,0.9999,pi,0.9999,pi,0.9999,pi,0.9999,pi,0.9999)
mlres <- nlminb(svals, function(x) nllfn(x,q,theta1,theta2),lower=clo, upper=chi)
maxi <- -mlres[[2]] ; pests <- mlres[[1]]

multC <- pests[1] ; rho1C <- pests[2] ; rho2C <- pests[3]
gammax <- maxgam(rho1C,rho2C) ; gamC <- multC*gammax

mu1 <- Arg(exp((1i)*pests[4])) ; gam1 <- pests[5] 
phi1 <- Arg(exp((1i)*pests[6])) ; omega1 <- pests[7]
alpha21 <- gam1^2+gam1*(1-gam1)*omega1*cos(phi1) ; beta21 <- gam1*(1-gam1)*omega1*sin(phi1)

mu2 <- Arg(exp((1i)*pests[8])) ; gam2 <- pests[9]
phi2 <- Arg(exp((1i)*pests[10])) ; omega2 <- pests[11]
alpha22 <- gam2^2+gam2*(1-gam2)*omega2*cos(phi2) ; beta22 <- gam2*(1-gam2)*omega2*sin(phi2)

mleres <- c(maxi,gamC,rho1C,rho2C,mu1,gam1,alpha21,beta21,mu2,gam2,alpha22,beta22)
return(mleres)
}

# -----------------------------------------------------------------------------

MLres <- Mod4KJ4ML(theta1, theta2, KJ4MLT1, KJ4MLT2, Mod4MLEs) ; MLres

maxll <- MLres[1] ; npar <- 11
AIC <- 2*npar-2*maxll ; AIC
BIC <- npar*log(n)-2*maxll ; BIC

# -----------------------------------------------------------------------------
Mod4KJden <- function(circarg, den1arg, den2arg, t1, t2) {
# -----------------------------------------------------------------------------

# Computes values of bivariate circular density with Kato-Jones 
# marginal circular densities and a Model 5 circula density. 

q <- circarg[1] ; gammac <- circarg[2] ; rho1c <- circarg[3] ; rho2c <- circarg[4]
mu1 <- den1arg[1] ; gamma1 <- den1arg[2] ; alpha21 <- den1arg[3] ; beta21 <- den1arg[4] 
rho1 <- sqrt(alpha21^2+beta21^2)/gamma1 ; lambda1 <- atan2(beta21,alpha21)
mu2 <- den2arg[1] ; gamma2 <- den2arg[2] ; alpha22 <- den2arg[3] ; beta22 <- den2arg[4] 
rho2 <- sqrt(alpha22^2+beta22^2)/gamma2 ; lambda2 <- atan2(beta22,alpha22)

nt <- length(t1) ; denind <- 0 ; den <- 0
p1 <- KJPDF(t1, mu1, rho1, gamma1, lambda1) ; arg1 <- CircVals(t1, den1arg)
p2 <- KJPDF(t2, mu2, rho2, gamma2, lambda2) ; arg2 <- CircVals(t2, den2arg)
for (i in 1:nt) { 
for (j in 1:nt) { denind <- denind+1
arg <- cos(arg1[j]-q*arg2[i]) ; carg1 <- cos(arg1[j]) ; carg2 <- cos(arg2[i])
circden <- 1+2*gammac*(arg-rho2c*carg1-rho1c*carg2+rho1c*rho2c)/((1+rho1c^2-2*rho1c*carg1)*(1+rho2c^2-2*rho2c*carg2))
den[denind] <- p1[j]*p2[i]*circden
} }
return(den) 
}

# -----------------------------------------------------------------------------
# Contour plot of ML fitted bivariate circular density with data superimposed. 
# -----------------------------------------------------------------------------

circML <- c(q,MLres[2],MLres[3],MLres[4])
mu1ML <- MLres[5] ; gam1ML <- MLres[6] ; a21ML <- MLres[7] ; b21ML <- MLres[8]
den1ML <- c(mu1ML, gam1ML, a21ML, b21ML)
mu2ML <- MLres[9] ; gam2ML <- MLres[10] ; a22ML <- MLres[11] ; b22ML <- MLres[12]
den2ML <- c(mu2ML, gam2ML, a22ML, b22ML)

t1val <- seq(-pi, pi, 2*pi/800) ; t2val <- seq(-pi, pi, 2*pi/800)
den <- Mod4KJden(circML, den1ML, den2ML, t1val, t2val)

par(lab=c(6,6,1), font.main=1, cex.axis=1.2, mai=c(0.45,0.45,0.05,0.05))
contour(seq(-pi, pi, 2*pi/800), seq(-pi, pi, 2*pi/800), matrix(den,801,801), xlim=c(-pi, pi), ylim=c(-pi, pi), xaxt="n", yaxt="n", lty=1, lwd=2, col="darkblue", nlevels = 20, cex.lab=1.0, labcex=0.8)
axis(1, at=seq(-pi, pi, pi/2), labels=c(expression(-pi),expression(-pi/2),0,expression(pi/2),expression(pi)))
axis(2, at=seq(-pi, pi, pi/2), labels=c(expression(-pi),expression(-pi/2),0,expression(pi/2),expression(pi)))
points(theta1, theta2, pch=16, cex=0.6, col="red")

maxden <- max(den) ; maxden

# -----------------------------------------------------------------------------
# Scatterplot of the pseudo-sample using the ML estimates of the marginal 
# distributions from the full bivariate circular model fit.
# -----------------------------------------------------------------------------

cang1 <- CircVals(theta1, den1ML) ; cang2 <- CircVals(theta2, den2ML)

par(mfrow=c(1,1), lab=c(6,6,1), font.main=1, cex.axis=1.2, mai=c(0.45,0.45,0.05,0.05))
plot(cang1, cang2, pch=16, cex=0.6, col="red", xlim=c(-pi, pi), ylim=c(-pi, pi), xaxt="n", yaxt="n")
axis(1, at=seq(-pi, pi, pi/2), labels=c(expression(-pi),expression(-pi/2),0,expression(pi/2),expression(pi)))
axis(2, at=seq(-pi, pi, pi/2), labels=c(expression(-pi),expression(-pi/2),0,expression(pi/2),expression(pi)))

# -----------------------------------------------------------------------------
# Contour plot of fitted Model 4 circula from the full model fit, with 
# pseudo-values superimposed.
# -----------------------------------------------------------------------------

mlgamC <- MLres[2] ; mlrho1C <- MLres[3] ; mlrho2C <- MLres[4]

t1val <- seq(-pi, pi, 2*pi/200) ; t2val <- seq(-pi, pi, 2*pi/200)
nt <- length(t1val) ; circind <- 0 ; circ <- 0
for (i in 1:nt) { t2 <- t2val[i] 
for (j in 1:nt) { t1 <- t1val[j] ; circind <- circind+1
circ[circind] <- Mod4cden(mlgamC, mlrho1C, mlrho2C, q, t1, t2)
} }

par(lab=c(6,6,1), font.main=1, cex.axis=1.2, mai=c(0.45,0.45,0.05,0.05))
contour(seq(-pi, pi, 2*pi/200), seq(-pi, pi, 2*pi/200), matrix(circ,201,201), xlim=c(-pi, pi), ylim=c(-pi, pi), xaxt="n", yaxt="n", lty=1, lwd=2, col="darkblue", nlevels = 10, cex.lab=1.0, labcex=0.8)
axis(1, at=seq(-pi, pi, pi/2), labels=c(expression(-pi),expression(-pi/2),0,expression(pi/2),expression(pi)))
axis(2, at=seq(-pi, pi, pi/2), labels=c(expression(-pi),expression(-pi/2),0,expression(pi/2),expression(pi)))
points(cang1, cang2, pch=16, cex=0.6, col="red")

# -----------------------------------------------------------------------------
# Calculation of MM estimate of rhoR, identification of q and bootstrap testing 
# for independence.
# -----------------------------------------------------------------------------

n <- length(cang1)
p1 <- abs(sum(exp((1i)*(cang1-cang2)))) ; p2 <- abs(sum(exp((1i)*(cang1+cang2))))
rhoRest <- (p1-p2)/n ; arhoRest <- abs(rhoRest) ; rhoRest
qval <- sign(rhoRest) ; qval

nind <- 1
B <- 999
for (i in 1:B) {
rcang2 <- sample(cang2,replace=F)
p1 <- abs(sum(exp((1i)*(cang1-rcang2)))) ; p2 <- abs(sum(exp((1i)*(cang1+rcang2))))
rhoRestB <- (p1-p2)/n
if (abs(rhoRestB) >= arhoRest) nind <- nind+1
}
pvalest <- nind/(B+1) ; pvalest

# -----------------------------------------------------------------------------
# Goodness-of-fit testing based on transformation to putative toroidal pseudo-
# uniformity
# -----------------------------------------------------------------------------

# -----------------------------------------------------------------------------
# Model 4 circula conditional densities.
# -----------------------------------------------------------------------------

# -----------------------------------------------------------------------------
Mod4cden1g2 <- function(theta1, theta2, gamma, rho1, rho2, q) {
# -----------------------------------------------------------------------------
arg <- cos(theta1-q*theta2)
cden <- (1+2*gamma*(arg-rho2*cos(theta1)-rho1*cos(theta2)+rho1*rho2)/((1+rho1^2-2*rho1*cos(theta1))*(1+rho2^2-2*rho2*cos(theta2))))/(2*pi)
return(cden) 
}

# -----------------------------------------------------------------------------
Mod4cden2g1 <- function(theta2, theta1, gamma, rho1, rho2, q) {
# -----------------------------------------------------------------------------
arg <- cos(theta1-q*theta2)
cden <- (1+2*gamma*(arg-rho2*cos(theta1)-rho1*cos(theta2)+rho1*rho2)/((1+rho1^2-2*rho1*cos(theta1))*(1+rho2^2-2*rho2*cos(theta2))))/(2*pi)
return(cden) 
}

# -----------------------------------------------------------------------------
# Computation of putative toroidal pseudo-uniform variates.
# -----------------------------------------------------------------------------

arg1 <- 0 ; arg2 <- 0 ; arg3 <- 0 ; arg4 <- 0
for (i in 1:n) {
arg1[i] <- cang1[i]%%(2*pi) ; arg2[i] <- cang2[i]%%(2*pi)
a1 <- arg1[i] ; a2 <- arg2[i]
arg3[i] <- 2*pi*integrate(function(x) Mod4cden1g2(x, a2, mlgamC, mlrho1C, mlrho2C, q),0,a1,abs.tol=1e-50)[[1]]
arg4[i] <- 2*pi*integrate(function(x) Mod4cden2g1(x, a1, mlgamC, mlrho1C, mlrho2C, q),0,a2,abs.tol=1e-50)[[1]]
arg1[i] <- Arg(exp((1i)*arg1[i])) ; arg2[i] <- Arg(exp((1i)*arg2[i]))
arg3[i] <- Arg(exp((1i)*arg3[i])) ; arg4[i] <- Arg(exp((1i)*arg4[i]))
}

# -----------------------------------------------------------------------------
# Scatterplots of putative toroidal pseudo-uniform variates.
# -----------------------------------------------------------------------------

par(lab=c(6,6,1), font.main=1, cex.axis=1.2, mai=c(0.45,0.45,0.05,0.05))
plot(arg2, arg3, pch=16, cex=0.6, col="red", xlim=c(-pi, pi), ylim=c(-pi, pi), xaxt="n", yaxt="n")
axis(1, at=seq(-pi, pi, pi/2), labels=c(expression(-pi),expression(-pi/2),0,expression(pi/2),expression(pi)))
axis(2, at=seq(-pi, pi, pi/2), labels=c(expression(-pi),expression(-pi/2),0,expression(pi/2),expression(pi)))

par(lab=c(6,6,1), font.main=1, cex.axis=1.2, mai=c(0.45,0.45,0.05,0.05))
plot(arg1, arg4, pch=16, cex=0.6, col="red", xlim=c(-pi, pi), ylim=c(-pi, pi), xaxt="n", yaxt="n")
axis(1, at=seq(-pi, pi, pi/2), labels=c(expression(-pi),expression(-pi/2),0,expression(pi/2),expression(pi)))
axis(2, at=seq(-pi, pi, pi/2), labels=c(expression(-pi),expression(-pi/2),0,expression(pi/2),expression(pi)))

# -----------------------------------------------------------------------------
WellnerBTStat <- function(arg1, arg2, arg3, arg4) {
# -----------------------------------------------------------------------------

# Computation of the maximum of the Wellner test statistic values for the 
# two sets of putative toroidal pseudo-uniform variates.

theta1 <- arg2 ; theta2 <- arg3
n <- length(theta1)
scc <- sum(cos(theta1)*cos(theta2)) ; scs <- sum(cos(theta1)*sin(theta2))
ssc <- sum(sin(theta1)*cos(theta2)) ; sss <- sum(sin(theta1)*sin(theta2))
statval1 <- 4*(scc*scc+scs*scs+ssc*ssc+sss*sss)/n
theta1 <- arg1 ; theta2 <- arg4
scc <- sum(cos(theta1)*cos(theta2)) ; scs <- sum(cos(theta1)*sin(theta2))
ssc <- sum(sin(theta1)*cos(theta2)) ; sss <- sum(sin(theta1)*sin(theta2))
statval2 <- 4*(scc*scc+scs*scs+ssc*ssc+sss*sss)/n
statval <- max(statval1,statval2)
return(statval)
}

# -----------------------------------------------------------------------------

StatVals <- WellnerBTStat(arg1, arg2, arg3, arg4) ; StatVals

# -----------------------------------------------------------------------------
Mod4KJSim <- function(circarg, den1arg, den2arg, maxden, n) {
# -----------------------------------------------------------------------------

# Simulation from bivariate circular model with Kato-Jones marginal circular 
# densities and a Model 4 circula density, using simple acceptance-rejection. 

theta1 <- 0 ; theta2 <- 0
for (i in 1:n) {
stopgo <- 1
while (stopgo == 1) {
t1 <- 2*pi*runif(1,0,1)-pi ; t2 <- 2*pi*runif(1,0,1)-pi
boxh <- maxden*runif(1,0,1)
dval <- Mod4KJden(circarg, den1arg, den2arg, t1, t2)
if (boxh <= dval) {
stopgo <- 0
theta1[i] <- t1 ; theta2[i] <- t2
} } } 
return(list(theta1,theta2))
}

# -----------------------------------------------------------------------------
Mod4cSim <- function(gamma, rho1, rho2, q, n) {
# -----------------------------------------------------------------------------

# Simulation from Model 4 circula using simple acceptance-rejection. 

cmax <- (1+2*gamma*(1-rho2-rho1+rho1*rho2)/((1+rho1^2-2*rho1)*(1+rho2^2-2*rho2)))/(4*pi^2) ; theta1 <- 0 ; theta2 <- 0
for (i in 1:n) {
stopgo <- 1
while (stopgo == 1) {
t1 <- 2*pi*runif(1,0,1)-pi ; t2 <- 2*pi*runif(1,0,1)-pi
boxh <- cmax*runif(1,0,1)
dval <- Mod4cden(gamma, rho1, rho2, q, t1, t2) 
if (boxh <= dval) {
stopgo <- 0
theta1[i] <- t1 ; theta2[i] <- t2
} } } 
return(list(theta1,theta2))
}

# -----------------------------------------------------------------------------
BootGOF <- function(circML, den1ML, den2ML, maxden, n, B, StatVals) {
# -----------------------------------------------------------------------------

# Bootstrap goodness-of-fit testing based on the maximum of the Wellner test 
# statistic values for two sets of toroidal pseudo-uniform variates.

nextrmB <- 1 ; q <- circML[1]

for (j in 2:(B+1)) {

SimRes <- Mod4KJSim(circML, den1ML, den2ML, maxden, n)
thet1 <- SimRes[[1]] ; thet2 <- SimRes[[2]]
MLres <- Mod4KJ4ML(thet1, thet2, den1ML, den2ML, circML)
mu1ML <- MLres[5] ; gam1ML <- MLres[6] ; a21ML <- MLres[7] ; b21ML <- MLres[8]
d1ML <- c(mu1ML, gam1ML, a21ML, b21ML)
mu2ML <- MLres[9] ; gam2ML <- MLres[10] ; a22ML <- MLres[11] ; b22ML <- MLres[12]
d2ML <- c(mu2ML, gam2ML, a22ML, b22ML)
cang1 <- CircVals(thet1, d1ML) ; cang2 <- CircVals(thet2, d2ML)

mlgamC <- MLres[2] ; mlrho1C <- MLres[3] ; mlrho2C <- MLres[4]
arg1 <- 0 ; arg2 <- 0 ; arg3 <- 0 ; arg4 <- 0
for (i in 1:n) {
arg1[i] <- cang1[i]%%(2*pi) ; arg2[i] <- cang2[i]%%(2*pi)
a1 <- arg1[i] ; a2 <- arg2[i]
arg3[i] <- 2*pi*integrate(function(x) Mod4cden1g2(x, a2, mlgamC, mlrho1C, mlrho2C, q),0,a1,abs.tol=1e-50)[[1]]
arg4[i] <- 2*pi*integrate(function(x) Mod4cden2g1(x, a1, mlgamC, mlrho1C, mlrho2C, q),0,a2,abs.tol=1e-50)[[1]]
arg1[i] <- Arg(exp((1i)*arg1[i])) ; arg2[i] <- Arg(exp((1i)*arg2[i]))
arg3[i] <- Arg(exp((1i)*arg3[i])) ; arg4[i] <- Arg(exp((1i)*arg4[i]))
}

StatVals[j] <- WellnerBTStat(arg1, arg2, arg3, arg4) 
if (StatVals[j] >= StatVals[1]) {nextrmB <- nextrmB+1}
}

pvalB <- nextrmB/(B+1) 
return(list(pvalB, StatVals))
}

# -----------------------------------------------------------------------------

B <- 99 ; n <- length(theta1)
BootGOFRes <- BootGOF(circML, den1ML, den2ML, maxden, n, B, StatVals)

BootGOFRes[[1]] ; tstatB <- BootGOFRes[[2]] ; tstatB
sort(tstatB)












