# ------------------------------------------------------------------------
# Initial part of bivariate circular KPJ analysis: fitting K-J marginals, 
# identifying q, and aids to bivariate circula model identification.
# Bivariate circular data assumed to be in theta1 and theta2 (in [-pi,pi)). 
# ------------------------------------------------------------------------

# ---------------------------------------------------------
# Initial scatterplot of data
# ---------------------------------------------------------

par(mfrow=c(1,1), lab=c(6,6,1), font.main=1, cex.axis=1.2, mai=c(0.45,0.45,0.05,0.05))
plot(theta1, theta2, pch=16, cex=1, col="red", xlim=c(-pi, pi), ylim=c(-pi, pi), xaxt="n", yaxt="n")
axis(1, at=seq(-pi, pi, pi/2), labels=c(expression(-pi),expression(-pi/2),0,expression(pi/2),expression(pi)))
axis(2, at=seq(-pi, pi, pi/2), labels=c(expression(-pi),expression(-pi/2),0,expression(pi/2),expression(pi)))


# ---------------------------------------------------------
KJPDF <- function(theta, mu, rho, gamma, lambda) { 
# ---------------------------------------------------------

# Calculates value of Kato-Jones probability density function.

# Assumes both mu and theta in [-pi,pi). Parameters rho,
# gamma and lambda assumed to be within their valid ranges.

den <- (cos(theta-mu)-rho*cos(lambda))/(1+rho*rho-2*rho*cos(theta-mu-lambda))
den <- (1+2*gamma*den)/(2*pi)
return(den) 
}

# ---------------------------------------------------------
KJAM <- function(mu, rho, gamma, lambda) {
# ---------------------------------------------------------

# Calculates the antimode for a Kato-Jones distribution.

if ( sin(lambda) == 0 ) {
AM <- mu+pi
} else {
v1 <- mu+lambda+atan((1-rho*rho)/((1+rho*rho)*tan(lambda)))
v2 <- -acos((2*rho*sin(lambda))/sqrt(((1+rho*rho)**2)-4*rho*rho*((cos(lambda))**2)))
if (sin(lambda) > 0) {
AM <- v1-v2
} else if (sin(lambda) < 0) {
AM <- v1-v2+pi
} }
AM <- Arg(exp((1i)*AM))
if (AM == pi) AM <- -pi
return(AM)
}

# ---------------------------------------------------------
KJDF <- function(theta, rho, gamma, lambda) {
# ---------------------------------------------------------

# Calculates DF of Kato-Jones distribution using the formulae in the 
# Supplementary Material to Kato and Jones (2015) and -pi as the 
# reference point (corresponding to mu=0). 
# All values of theta are assumed to be in [-pi,pi).

if (theta == -pi) {
df <- 0
} else if (rho == 0) {
df <- (theta+pi+2*gamma*sin(theta))/(2*pi)
} else if ( (rho > 0) && ((lambda == 0)||(lambda == -pi)) ) {
rhot <- rho*cos(lambda)
arg <- ((1+rhot)/(1-rhot))*tan(theta/2)
df <- ( (1-gamma/rhot)*(theta+pi) + (gamma*pi/rhot) + (2*gamma/rhot)*(atan(arg)) )/(2*pi)
} else {
ind <- 0
if ( tan((theta-lambda)/2) < 1/tan(lambda/2) ) {
ind <- 1
}
df <- (1-(gamma*cos(lambda)/rho))*(theta+pi)+(gamma*sin(lambda)/rho)*log( (1+rho^2+2*rho*cos(lambda))/(1+rho^2-2*rho*cos(theta-lambda)) )
arg <- atan( ((1+rho)/(1-rho))*tan((theta-lambda)/2) ) - atan( (1+rho)/((1-rho)*tan(lambda/2)) ) + pi*ind
df <- (df + (2*gamma*cos(lambda)/rho)*arg)/(2*pi)
}
return(df)
}
# ---------------------------------------------------------

# ---------------------------------------------------------
# FITTING KATO-JONES MARGINALS 
# ---------------------------------------------------------

# --------------------------------------------------------------------
KJ4MM <- function(theta) {
# --------------------------------------------------------------------

# Method of moments estimation for a Kato-Jones distribution
# Parametrization used is mu, gamma, alpha2, beta2

S1n <- mean(exp((1i)*theta)) ; mu_mm <- Arg(S1n) ; gamma_mm <- abs(S1n)
C2n <- mean(cos(2*(theta-mu_mm))) ; S2n <- mean(sin(2*(theta-mu_mm)))

if( (C2n-gamma_mm^2)^2+S2n^2 <= gamma_mm^2*(1-gamma_mm)^2 ){
alpha2_mm <- C2n ; beta2_mm <- S2n
} else {
alpha2_mm <- gamma_mm*(1-gamma_mm)*(C2n-gamma_mm^2)/sqrt((C2n-gamma_mm^2)^2+S2n^2)+gamma_mm^2
beta2_mm <- gamma_mm*(1-gamma_mm)*S2n/sqrt((C2n-gamma_mm^2)^2+S2n^2)
}
mmest <- c(mu_mm,gamma_mm,alpha2_mm,beta2_mm)
return(mmest)
}

# --------------------------------------------------------------------

KJ4MMT1 <- KJ4MM(theta1) ; KJ4MMT2 <- KJ4MM(theta2)
# MM estimates
# Parametrization used is mu, gamma, alpha2, beta2
KJ4MMT1 ; KJ4MMT2

# --------------------------------------------------------------------
KJ4Conv <- function(parsin) {
# --------------------------------------------------------------------

# Converts values of alpha2 and beta2 to values of phi and omega.

mu <- parsin[1] ; gamma <- parsin[2] ; alpha2 <- parsin[3] ; beta2 <- parsin[4]
phi <- atan2(beta2, (alpha2-gamma^2)) ; omega <- beta2/(gamma*(1-gamma)*sin(phi))
parsout <- c(mu, gamma, phi, omega)
return(parsout)
}

# --------------------------------------------------------------------

KJ4MMT1 <- KJ4Conv(KJ4MMT1) ; KJ4MMT1
if (KJ4MMT1[4] > 0.99) KJ4MMT1[4] <- 0.98
KJ4MMT1

KJ4MMT2 <- KJ4Conv(KJ4MMT2) ; KJ4MMT2
if (KJ4MMT2[4] > 0.99) KJ4MMT2[4] <- 0.98
KJ4MMT2

# --------------------------------------------------------------------
KJ4ML <- function(theta, KJ4MM) {
# --------------------------------------------------------------------

# Maximum likelihood estimation for Kato-Jones distribution.
# MM estimates of mu, gamma, phi and omega inputted.
# ML estimates of mu, gamma, alpha2 and beta2 outputted.

nll <- function(p, theta) {
mu <- p[1] ; gamma <- p[2] ; phi <- p[3] ; omega <- p[4]
n <- length(theta)
alpha2m <- gamma*(gamma+(1-gamma)*omega*cos(phi)) ; beta2m <- gamma*(1-gamma)*omega*sin(phi)
ll <- sum(log(1+2*gamma^2*(gamma*cos(theta-mu)-alpha2m)/(gamma^2+alpha2m^2+beta2m^2-2*gamma*(alpha2m*cos(theta-mu)+beta2m*sin(theta-mu)))))
return(-ll+n*log(2*pi))
}

try <- nlminb(KJ4MM, function(x) nll(x,theta),lower=c(-pi,0,-pi,0),upper=c(pi-1e-6,1-1e-6,pi-1e-6,1-1e-6))
pest <- try[[1]]
mu_ml <- Arg(exp((1i)*pest[1])) ; gamma_ml <- pest[2]
phi_ml <- Arg(exp((1i)*pest[3])) ; omega_ml <- pest[4]
alpha2_ml <- gamma_ml^2+gamma_ml*(1-gamma_ml)*omega_ml*cos(phi_ml)
beta2_ml <- gamma_ml*(1-gamma_ml)*omega_ml*sin(phi_ml)
mlest <- c(mu_ml,gamma_ml,alpha2_ml,beta2_ml)
return(mlest)
}

# --------------------------------------------------------------------

KJ4MLT1 <- KJ4ML(theta1, KJ4MMT1) ; KJ4MLT2 <- KJ4ML(theta2, KJ4MMT2)
# ML estimates
# Parametrization used is mu, gamma, alpha2, beta2
KJ4MLT1 ; KJ4MLT2

# --------------------------------------------------------------------
HistKJfits <- function(theta1, theta2, KJ4MLT1, KJ4MLT2) {
# --------------------------------------------------------------------

# Histogram of data on each circular variable with fitted Kato-Jones
# density superimposed.

par(mfrow=c(2,1),mai=c(0.85,0.85,0.05,0.05),cex.axis=0.9)

MLEs <- KJ4MLT1
mu <- MLEs[1] ; gamma <- MLEs[2] ; alpha2 <- MLEs[3] ; beta2 <- MLEs[4]
rho <- sqrt(alpha2^2+beta2^2)/gamma ; lambda <- atan2(beta2,alpha2)
tval <- seq(-pi,pi,pi/200) ; denvals1 <- KJPDF(tval, mu, rho, gamma, lambda)
maxden <- max(denvals1)

MLEs <- KJ4MLT2
mu <- MLEs[1] ; gamma <- MLEs[2] ; alpha2 <- MLEs[3] ; beta2 <- MLEs[4]
rho <- sqrt(alpha2^2+beta2^2)/gamma ; lambda <- atan2(beta2,alpha2)
denvals2 <- KJPDF(tval, mu, rho, gamma, lambda)
maxden <- max(denvals2,maxden)

hist(theta1, probability=T, main="", breaks=seq(-pi,pi,2*pi/8), col="blue", xlab=expression(theta[1]), ylim=c(0,maxden+0.05), xaxt="n")
axis(1, at=seq(-pi, pi, pi/2), labels=c(expression(-pi),expression(-pi/2),0,expression(pi/2),expression(pi)))
lines(tval, denvals1, lty=2, col="red", lwd=2)

hist(theta2, probability=T, main="", breaks=seq(-pi,pi,2*pi/8), col="blue", xlab=expression(theta[2]), ylim=c(0,maxden+0.05), xaxt="n")
axis(1, at=seq(-pi, pi, pi/2), labels=c(expression(-pi),expression(-pi/2),0,expression(pi/2),expression(pi)))
lines(tval, denvals2, lty=2, col="red", lwd=2)
}

# --------------------------------------------------------------------

HistKJfits(theta1, theta2, KJ4MLT1, KJ4MLT2)

# --------------------------------------------------------------------
# CALCULATION AND SCATTERPLOT OF PSEUDO-SAMPLES (Using antimode as origen)
# --------------------------------------------------------------------

# ---------------------------------------------------------
CircVals <- function(T, KJ4MLT) {
# ---------------------------------------------------------

MLEs <- KJ4MLT
mu <- MLEs[1] ; gamma <- MLEs[2] ; alpha2 <- MLEs[3] ; beta2 <- MLEs[4]
rho <- sqrt(alpha2^2+beta2^2)/gamma ; lambda <- atan2(beta2,alpha2)
AM <- KJAM(mu, rho, gamma, lambda)
AMs <- Arg(exp((1i)*(AM-mu)))
if (AMs == pi) AMs <- -pi
GAMs <- KJDF(AMs, rho, gamma, lambda)
n <- length(T) ; cang <- 0
for (i in 1:n) {
ang <- T[i]
angs <- Arg(exp((1i)*(ang-mu))) 
if (angs == pi) angs <- -pi
if (angs >= AMs) {
cang[i] <- 2*pi*(KJDF(angs, rho, gamma, lambda) - GAMs)
} else if (angs < AMs) {
cang[i] <- 2*pi*(1 + KJDF(angs, rho, gamma, lambda) - GAMs)
}
cang[i] <- Arg(exp((1i)*cang[i]))
}
return(cang)
}

# ---------------------------------------------------------

cang1 <- CircVals(theta1, KJ4MLT1) ; cang2 <- CircVals(theta2, KJ4MLT2)

par(mfrow=c(1,1), lab=c(6,6,1), font.main=1, cex.axis=1.2, mai=c(0.45,0.45,0.05,0.05))
plot(cang1, cang2, pch=16, cex=1, col="red", xlim=c(-pi, pi), ylim=c(-pi, pi), xaxt="n", yaxt="n")
axis(1, at=seq(-pi, pi, pi/2), labels=c(expression(-pi),expression(-pi/2),0,expression(pi/2),expression(pi)))
axis(2, at=seq(-pi, pi, pi/2), labels=c(expression(-pi),expression(-pi/2),0,expression(pi/2),expression(pi)))

# --------------------------------------------------------------------
# Calculation of MM estimate of rhoR, identification of q and 
# bootstrap testing for independence.
# --------------------------------------------------------------------

n <- length(cang1)
p1 <- abs(sum(exp((1i)*(cang1-cang2)))) ; p2 <- abs(sum(exp((1i)*(cang1+cang2))))
rhoRest <- (p1-p2)/n ; arhoRest <- abs(rhoRest) ; rhoRest
q <- sign(rhoRest) ; q

nind <- 1
B <- 999
for (i in 1:B) {
rcang2 <- sample(cang2,replace=F)
p1 <- abs(sum(exp((1i)*(cang1-rcang2)))) ; p2 <- abs(sum(exp((1i)*(cang1+rcang2))))
rhoRestB <- (p1-p2)/n
if (abs(rhoRestB) >= arhoRest) nind <- nind+1
}
pvalest <- nind/(B+1) ; pvalest

# --------------------------------------------------------------------
# Lattice plots of empirical Fourier coefficients (for absolute, real and imaginary values).
# --------------------------------------------------------------------

library(RColorBrewer) ; library(lattice)
my_palette <- colorRampPalette(c("white", "darkblue"))(n = 200)

n <- length(cang1)

mmax <- 6 ; nmax <- 6 ; empcoef <- 0 ; ecind <- 0 
for (nval in 1:nmax) {  
for (mval in 1:mmax) { ecind <- ecind+1
empcoef[ecind] <- sum(exp((1i)*(mval*cang1-q*nval*cang2)))/n  
} }
mval <- seq(1,mmax) ; nval <- -q*seq(1,nmax)
data <- expand.grid(X=mval, Y=nval)

data$Z <- Mod(empcoef)
levelplot(Z ~ X*Y, data=data, xlab=expression(r), ylab=expression(-qs), col.regions = my_palette, main="") 
data$Z

data$Z <- Re(empcoef)
levelplot(Z ~ X*Y, data=data, xlab=expression(r), ylab=expression(-qs), col.regions = my_palette, main="") 
data$Z 

data$Z <- Im(empcoef)
levelplot(Z ~ X*Y, data=data, xlab=expression(r), ylab=expression(-qs), col.regions = my_palette, main="") 
data$Z

# --------------------------------------------------------------------
# End of initial part of KPJ analysis.
# --------------------------------------------------------------------

